# ============================================================
# LetterTemplate.py
# template class for LETTERS
# ------------------------------------------------------------
# Copyright (C) 2004 the Kumula Team.
# ============================================================

from qt import QString,QStringList
try:
    from reportlab.lib.styles import ParagraphStyle
    from reportlab.lib.colors import Color
except:
    pass

# --------------------------------------------------------


class KLetterTemplate:

    def __init__(self):
        self.metainfo = {}
        self.recinfo = QStringList()

    def title(self):
        # gets the name of this template (as shown in Letter's dropdown list)
        # OVERRIDE IT!
        name = "ABSTRACT CLASS"
        return name


    def wantedInfos(self):
        # tells LETTERS the wanted meta infos
        # array of array of: ID, Name, Type
        # Possible types: edit, date, int, float.X
        # OVERRIDE IT!
        infos = (
            ("abstract", "ABSTRACT CLASS", "edit"),
        )
        return infos


    def setInfo(self, aID, aValue):
        self.metainfo[aID] = aValue

    def setReceiver(self, aRec):
        self.recinfo = aRec


    def pageSize(self):
        # gets the preferred page size (Letter, A4, ...)
        # OVERRIDE IT TO CUSTOMIZE
        from reportlab.lib.pagesizes import Letter
        return Letter


    def formatReceiver(self, aAdTitle, aAddressee, aAdComment, aAddress, aZip, aCity, aDistrict, useContact, aContactTitle, aContactName, aContactForename):
        # takes the data from the selected client/contact person
        # and generates a standard receiver's field
        # OVERRIDE IT TO CUSTOMIZE
        myText = QString("")
        if (useContact == 1):
            if (aContactTitle != ""):
                myText.append(aContactTitle)
                myText.append("\n")
            if (aContactForename != ""):
                myText.append(aContactForename)
                myText.append(" ")
            if (aContactName != ""):
                myText.append(aContactName)
                myText.append("\n")
            if (aAddressee != ""):
                myText.append("c/o ")
                myText.append(aAddressee)
                myText.append("\n")
        else:
            if (aAdTitle != ""):
                myText.append(aAdTitle)
                myText.append("\n")
            if (aAddressee != ""):
                myText.append(aAddressee)
                myText.append("\n")

        if (aAdComment != ""):
            myText.append(aAdComment)
            myText.append("\n")
        if (aAddress != ""):
            myText.append(aAddress)
            myText.append("\n")
        if (aZip != ""):
            myText.append("\n")
            myText.append(aZip)
            myText.append(" ")
        if (aCity != ""):
            myText.append(aCity)
        if (aDistrict != ""):
            myText.append("-")
            myText.append(aDistrict)
            myText.append("\n")
        return myText



    def textStyle(self):
        # OVERRIDE IT TO CHANGE FONT, SIZE etc.
        myStyle = ParagraphStyle("myBodyStyle")
        myStyle.fontName = "Helvetica"
        myStyle.fontSize = 10
        myStyle.textColor = Color(0,0,0)
        myStyle.spaceAfter = 5
        return myStyle


    def formatBody(self, aBodyText):
        # OVERRIDE TO CHANGE THE WAY THE TEXT IS FORMATTED
        from reportlab.lib.units import mm
        from reportlab.platypus import Paragraph,Spacer,NextPageTemplate

        Story = []
        style = self.textStyle()
        paraList = QStringList.split("<p>", aBodyText, 1)
        for lineNr in range(0, paraList.count()):
            line = paraList[lineNr]
            line.remove("</p>")
            p = Paragraph(line.ascii(), style)
            Story.append(p)
            if (lineNr == 0):
                Story.append(NextPageTemplate('LaterPage'))
        return Story


    def doFirstPage(self, canvas, doc):
        # OVERRIDE IT TO FORMAT THE FIRST PAGE (to draw images etc.)
        pass

    def doLaterPages(self, canvas, doc):
        # OVERRIDE IT TO FORMAT THE LATER PAGES (to draw images etc.)
        pass

    def firstFrames(self):
        # OVERRIDE IT TO DEFINE THE FRAME(S) OF THE FIRST PAGE
        from reportlab.platypus import Frame
        from reportlab.lib.units import mm
        firstFrames = [Frame(20*mm, 20*mm, 140*mm, 200*mm, leftPadding=0, rightPadding=0, topPadding=0, bottomPadding=0)]
        return firstFrames

    def laterFrames(self):
        # OVERRIDE IT TO DEFINE THE FRAME(S) OF THE LATER PAGES
        from reportlab.platypus import Frame
        from reportlab.lib.units import mm
        laterFrames = [Frame(20*mm, 20*mm, 170*mm, 260*mm, leftPadding=0, rightPadding=0, topPadding=0, bottomPadding=0)]
        return laterFrames


    def buildLetter(self, aFileName, aBodyText):
        # most important method for building the PDF
        # doesn't need to be overwritten normally
        from reportlab.platypus import BaseDocTemplate,PageTemplate
        doc = BaseDocTemplate(aFileName, pagesize=self.pageSize(), showBoundary=0)
        pageTmpl = [PageTemplate(id='FirstPage', frames=self.firstFrames(), onPage=self.doFirstPage),
                    PageTemplate(id='LaterPage', frames=self.laterFrames(), onPage=self.doLaterPages)]
        doc.addPageTemplates(pageTmpl)
        Story = self.formatBody(aBodyText)
        doc.build(Story)

