# ============================================================
# MainWindow.py
# main window for the letter assistent.
# ------------------------------------------------------------
# Copyright (C) 2004 the Kumula Team.
# ============================================================

import sys
import os
import tempfile
import kumula
from qt import *
from KumulaMainWindow import KKumulaMainWindow,KAction
from SpecialItems import KListViewItemWithID

from LetterTemplate import KLetterTemplate

# ------------------------------------------------------------

try:
    from reportlab.platypus import SimpleDocTemplate, Paragraph, Spacer
    from reportlab.lib.styles import getSampleStyleSheet
    styles = getSampleStyleSheet()
except:
    pass


class KMainWindow(KKumulaMainWindow):


    def getProgramName(self):
        return "Letters"

    def getProgramVersion(self):
        return "0.3"


    def createUI(self):
        KKumulaMainWindow.createUI(self)

        myTitle = QString("Letter to ")
        myTitle.append(kumula.ClientName)
        myTitle.append(" (")
        myTitle.append(kumula.ClientID)
        myTitle.append(")")
        self.setCaption(myTitle)

        self.statusBar()
        self.tempFiles = ()
        # initalize user interface

        self.setCentralWidget(QWidget(self,"qt_central_widget"))
        MainWindowLayout = QGridLayout(self.centralWidget(),1,1,0,0,"MainWindowLayout")

        self.tabMain = QTabWidget(self.centralWidget(),"tabMain")
        self.tab = QWidget(self.tabMain,"tab")
        tabLayout = QHBoxLayout(self.tab,0,0,"tabLayout")

        self.lvAddresses = QListView(self.tab,"lvAddresses")
        self.lvAddresses.addColumn(self.__tr("Addresses and contacts"))
        self.lvAddresses.setAllColumnsShowFocus(1)
        self.lvAddresses.setRootIsDecorated(1)
        self.lvAddresses.setResizeMode(QListView.LastColumn)
        tabLayout.addWidget(self.lvAddresses)

        self.edReceiver = QTextEdit(self.tab,"edReceiver")
        self.edReceiver.setWordWrap(QTextEdit.WidgetWidth)
        tabLayout.addWidget(self.edReceiver)
        self.tabMain.insertTab(self.tab,QString(""))

        self.tab_2 = QWidget(self.tabMain,"tab_2")
        tabLayout_2 = QGridLayout(self.tab_2,1,1,11,6,"tabLayout_2")
        self.gbValues = QGroupBox(self.tab_2,"gbValues")
        self.gbValues.setColumnLayout(0,Qt.Vertical)
        self.gbValues.layout().setSpacing(6)
        self.gbValues.layout().setMargin(11)

#        gbValuesLayout = QGridLayout(self.gbValues.layout())
#        gbValuesLayout.setAlignment(Qt.AlignTop)
#        self.cbMySign = QCheckBox(self.gbValues,"cbMySign")
#        gbValuesLayout.addWidget(self.cbMySign,2,0)
#        self.edMySign = QLineEdit(self.gbValues,"edMySign")
#        gbValuesLayout.addWidget(self.edMySign,2,1)
#        self.edAddress = QLineEdit(self.gbValues,"edAddress")
#        gbValuesLayout.addWidget(self.edAddress,4,1)
#        self.cbAddress = QCheckBox(self.gbValues,"cbAddress")
#        gbValuesLayout.addWidget(self.cbAddress,4,0)
#        self.cbMyDate = QCheckBox(self.gbValues,"cbMyDate")
#        gbValuesLayout.addWidget(self.cbMyDate,3,0)
#        self.edMyDate = QDateEdit(self.gbValues,"edMyDate")
#        gbValuesLayout.addWidget(self.edMyDate,3,1)
#        self.cbSenderDate = QCheckBox(self.gbValues,"cbSenderDate")
#        gbValuesLayout.addWidget(self.cbSenderDate,1,0)
#        self.edSenderDate = QDateEdit(self.gbValues,"edSenderDate")
#        gbValuesLayout.addWidget(self.edSenderDate,1,1)
#        self.cbSenderSign = QCheckBox(self.gbValues,"cbSenderSign")
#        gbValuesLayout.addWidget(self.cbSenderSign,0,0)
#        self.edSenderSign = QLineEdit(self.gbValues,"edSenderSign")
#        gbValuesLayout.addWidget(self.edSenderSign,0,1)

        tabLayout_2.addMultiCellWidget(self.gbValues,1,1,0,1)
        self.cbTemplate = QComboBox(0,self.tab_2,"cbTemplate")
        tabLayout_2.addWidget(self.cbTemplate,0,1)
        self.lblTemplate = QLabel(self.tab_2,"lblTemplate")
        self.lblTemplate.setAlignment(QLabel.AlignVCenter | QLabel.AlignRight)
        tabLayout_2.addWidget(self.lblTemplate,0,0)
        self.tabMain.insertTab(self.tab_2,QString(""))

        self.TabPage = QWidget(self.tabMain,"TabPage")
        TabPageLayout = QGridLayout(self.TabPage,1,1,0,0,"TabPageLayout")
        self.edBody = QTextEdit(self.TabPage,"edBody")
        self.edBody.setTextFormat(Qt.RichText)
        self.edBody.setTabChangesFocus(1)
        TabPageLayout.addWidget(self.edBody,0,0)
        self.tabMain.insertTab(self.TabPage,QString(""))

        MainWindowLayout.addWidget(self.tabMain,0,0)

        self.connect(self.lvAddresses, SIGNAL("selectionChanged()"), self.updateReceiver)
        self.connect(self.cbTemplate, SIGNAL("activated(int)"), self.updateTemplate)
        self.connect(self.tabMain, SIGNAL("currentChanged(QWidget*)"), self.handleTabChanged)
        self.connect(self.edBody, SIGNAL("cursorPositionChanged(int,int)"), self.updateFormatButtons)

        self.languageChange()
        self.getAddressesFromDB()

        self.resize(QSize(450,380).expandedTo(self.minimumSizeHint()))
        self.clearWState(Qt.WState_Polished)
        self.setTabOrder(self.tabMain,self.lvAddresses)
        self.setTabOrder(self.lvAddresses,self.edReceiver)
        self.setTabOrder(self.edReceiver,self.cbTemplate)
        self.setTabOrder(self.cbTemplate,self.edBody)
#        self.setTabOrder(self.cbSenderSign,self.edSenderSign)
#        self.setTabOrder(self.edSenderSign,self.cbSenderDate)
#        self.setTabOrder(self.cbSenderDate,self.edSenderDate)
#        self.setTabOrder(self.edSenderDate,self.cbMySign)
#        self.setTabOrder(self.cbMySign,self.edMySign)
#        self.setTabOrder(self.edMySign,self.cbMyDate)
#        self.setTabOrder(self.cbMyDate,self.edMyDate)
#        self.setTabOrder(self.edMyDate,self.cbAddress)
#        self.setTabOrder(self.cbAddress,self.edAddress)
#        self.setTabOrder(self.edAddress,self.edBody)




    def createMenuBar(self):
        KKumulaMainWindow.createMenuBar(self)

        # initialize Actions
        self.actFileNew          = KAction(self, "file_new",       "New (this client)",        self.handleFileNew,          "Start a new letter for this client.")
        self.actFileNewClient    = KAction(self, "letter_new",     "New (another client)...",  self.handleFileNewClient,    "Start a new letter for another client.")
        self.actFileOpen         = KAction(self, "file_open",      "Open...",                  None,                        "Open an already written letter.")
        self.actFileSave         = KAction(self, "file_save",      "Save",                     None,                        "Save this letter.")
        self.actFileSaveAs       = KAction(self, "file_saveas",    "Save as...",               None,                        "Save this letter to another file.")
        self.actPdfShow          = KAction(self, "pdf_show",       "Show PDF",                 self.handlePdfShow,          "Generate a PDF file and show it.")
        self.actPdfSaveToFile    = KAction(self, "pdf_savetofile", "Save PDF to disk...",      None,                        "Generate a PDF file and save it to disk.")
        self.actPdfSaveToDMS     = KAction(self, "pdf_savetodms",  "Save PDF to documents...", None,                        "Generate a PDF file and save it to Kumula's document database.")
        self.actEditUndo         = KAction(self, "edit_undo",      "Undo",                     self.handleEditUndo,         "Undo the last change.",                        "Ctrl+Z")
        self.actEditRedo         = KAction(self, "edit_redo",      "Redo",                     self.handleEditRedo,         "Redo the last undo action.",                   "Ctrl+Shift+Z")
        self.actEditCut          = KAction(self, "edit_cut",       "Cut",                      self.handleEditCut,          "Cut the selection to the clipboard.",          "Ctrl+X")
        self.actEditCopy         = KAction(self, "edit_copy",      "Copy",                     self.handleEditCopy,         "Copy the selection to the clipboard.",         "Ctrl+C")
        self.actEditPaste        = KAction(self, "edit_paste",     "Paste",                    self.handleEditPaste,        "Paste the clipboard's content into the text.", "Ctrl+V")
        self.actEditDelete       = KAction(self, "edit_delete",    "Delete",                   self.handleEditDelete,       "Delete current selection.",                    "Del")
        self.actEditSelectAll    = KAction(self, "edit_selectall", "Select all",               self.handleEditSelectAll,    "Select the whole text.",                       "Ctrl+A")
        self.actFormatBold       = KAction(self, "text_bold",      "Bold",                     self.handleFormatBold,       "Bold")
        self.actFormatItalic     = KAction(self, "text_italic",    "Italic",                   self.handleFormatItalic,     "Italic")
        self.actFormatUnderlined = KAction(self, "text_under",     "Underlined",               self.handleFormatUnderlined, "Underlined")
        self.actFormatTextColor  = KAction(self, "text_color",     "Text color...",            None,                        "Text color")

        self.actFormatBold.setToggleAction(1)
        self.actFormatItalic.setToggleAction(1)
        self.actFormatUnderlined.setToggleAction(1)

        # the EDIT and FORMAT actions are connected in createUI(),
        # because the editor widget doesn't exist yet (in createMenuBar())!

        # create Menu Structure
        self.menuFile = self.makePopupMenu()
        self.actFileNew.addToMenuBar(self.menuFile)
        self.actFileNewClient.addToMenuBar(self.menuFile)
        self.menuFile.insertSeparator()
        self.actFileOpen.addToMenuBar(self.menuFile)
        self.actFileSave.addToMenuBar(self.menuFile)
        self.actFileSaveAs.addToMenuBar(self.menuFile)
        self.menuFile.insertSeparator()
        self.actPdfShow.addToMenuBar(self.menuFile)
        self.actPdfSaveToFile.addToMenuBar(self.menuFile)
        self.actPdfSaveToDMS.addToMenuBar(self.menuFile)
        self.MenuBar.insertItem(QString("&Letter"), self.menuFile, 1)

        self.menuEdit = self.makePopupMenu()
        self.actEditUndo.addToMenuBar(self.menuEdit)
        self.actEditRedo.addToMenuBar(self.menuEdit)
        self.menuEdit.insertSeparator()
        self.actEditCut.addToMenuBar(self.menuEdit)
        self.actEditCopy.addToMenuBar(self.menuEdit)
        self.actEditPaste.addToMenuBar(self.menuEdit)
        self.actEditDelete.addToMenuBar(self.menuEdit)
        self.menuEdit.insertSeparator()
        self.actEditSelectAll.addToMenuBar(self.menuEdit)
        self.MenuBar.insertItem(QString("&Edit"), self.menuEdit, 2)

        self.menuFormat = self.makePopupMenu()
        self.actFormatBold.addToMenuBar(self.menuFormat)
        self.actFormatItalic.addToMenuBar(self.menuFormat)
        self.actFormatUnderlined.addToMenuBar(self.menuFormat)
        self.menuFormat.insertSeparator()
        self.actFormatTextColor.addToMenuBar(self.menuFormat)
        self.MenuBar.insertItem(QString("&Format"), self.menuFormat, 3)




    def createToolBar(self):
        KKumulaMainWindow.createToolBar(self)

        self.actFileNewClient.addToToolBar(self.ToolBar, "New letter")
        self.ToolBar.addSeparator()
        self.actPdfShow.addToToolBar(self.ToolBar, "Show PDF")
        self.ToolBar.addSeparator()
        self.actEditUndo.addToToolBar(self.ToolBar, "Undo")
        self.actEditRedo.addToToolBar(self.ToolBar, "Redo")
        self.ToolBar.addSeparator()
        self.actEditCut.addToToolBar(self.ToolBar, "Cut")
        self.actEditCopy.addToToolBar(self.ToolBar, "Copy")
        self.actEditPaste.addToToolBar(self.ToolBar, "Paste")
        self.actEditDelete.addToToolBar(self.ToolBar, "Delete")
        self.ToolBar.addSeparator()
        self.actFormatBold.addToToolBar(self.ToolBar, "Bold")
        self.actFormatItalic.addToToolBar(self.ToolBar, "Italic")
        self.actFormatUnderlined.addToToolBar(self.ToolBar, "Underline")
        self.actFormatTextColor.addToToolBar(self.ToolBar, "Color")



    def languageChange(self):
        KKumulaMainWindow.languageChange(self)

        self.tabMain.changeTab(self.tab,self.__tr("Receiver"))
        self.tabMain.changeTab(self.tab_2,self.__tr("Values"))
        self.tabMain.changeTab(self.TabPage,self.__tr("Letter"))
        self.gbValues.setTitle(self.__tr("Select values:"))
#        self.cbMySign.setText(self.__tr("Our sign:"))
#        self.cbAddress.setText(self.__tr("Our address:"))
#        self.cbMyDate.setText(self.__tr("Date:"))
#        self.cbSenderDate.setText(self.__tr("Your letter from:"))
#        self.cbSenderSign.setText(self.__tr("Your sign:"))
        self.loadTemplates()
        self.lblTemplate.setText(self.__tr("Template:"))


    def closeEvent(self, e):
        # delete previosly generated PDFs
        try:
            for fileName in self.tempFiles:
                os.unlink(fileName)
        except:
            pass
        KKumulaMainWindow.closeEvent(self, e)



    def loadTemplates(self):
        self.disconnect(self.cbTemplate, SIGNAL("activated(int)"), self.updateTemplate)
        myViewerDir = os.path.join(os.path.dirname(sys.argv[0]), "..", "lib", "letters")
        # get list of all subdirectories of KUMULADIR/lib/letters
        tmpDirList = ()
        osDirList = os.listdir(myViewerDir)
        osDirList.sort()
        for Dir in osDirList:
            newDir = os.path.join(myViewerDir, Dir)
            if os.path.isdir(newDir):
                tmpDirList = tmpDirList + (newDir,)
                sys.path.append(newDir)

        self.templates = {}
        # in each subdirectory, load the file "template.py"...
        for fi in tmpDirList:
            myFilePath = os.path.join(fi, "template.py")
            myTemplFile = open(myFilePath, "r")
            # ...compile it...
            exec myTemplFile in locals()
            myTemplFile.close()
            # ...and add it to our template list
            myTemplate = ltTemplate()
            myName = myTemplate.title()
            self.templates[myName] = myTemplate

        self.cbTemplate.clear()
        for tmpl in self.templates:
            self.cbTemplate.insertItem(QString(tmpl))
        self.connect(self.cbTemplate, SIGNAL("activated(int)"), self.updateTemplate)



    def selectedTemplate(self):
        myName = self.cbTemplate.currentText()
        return self.templates[myName.ascii()]


    def updateTemplate(self, myID):
        tmpl = self.selectedTemplate()
        print "Updating", tmpl.title()

        # update ValueGrid
#        for myChild in self.gbValues.children():
#            self.gbValues.removeChild(myChild)
        gbValuesLayout = QGridLayout(self.gbValues.layout())
        line = 0
        for metainfo in tmpl.wantedInfos():
            id    = metainfo[0]
            label = metainfo[1]
            type  = metainfo[2]
            print "Adding", id
            myLabel = QLabel(self.gbValues, "myLabel")
            myLabel.setText(label)
            gbValuesLayout.addWidget(myLabel, line, 0)
            if (type == "date"):
                myEdit = QDateEdit(self.gbValues, id)
            else:
                myEdit = QLineEdit(self.gbValues, id)
            gbValuesLayout.addWidget(myEdit, line, 1)
            line = line + 1
        # update TextBox



    def getAddressesFromDB(self):
        self.lvAddresses.clear()
        self.edReceiver.clear()
        quAddresses = kumula.DB.doSelect("clients_addresses", ("id","comment"), "comment", {"clientid":kumula.ClientID})
        while quAddresses.next():
            aItem = KListViewItemWithID(self.lvAddresses, quAddresses.value(0).toString())
            aItem.setText(0, quAddresses.value(1).toString())
            quContacts = kumula.DB.doSelect("clients_contacts", ("id","name","forename"), "name", {"addressid":quAddresses.value(0).toString()})
            while quContacts.next():
                displayedName = quContacts.value(1).toString()
                displayedName.append(" ")
                displayedName.append(quContacts.value(2).toString())
                cItem = KListViewItemWithID(aItem, quContacts.value(0).toString())
                cItem.setText(0, displayedName)
            aItem.setOpen(1)
        if self.lvAddresses.firstChild():
            self.lvAddresses.setSelected(self.lvAddresses.firstChild(), 1)


    def updateReceiver(self):
        myItem = self.lvAddresses.currentItem()
        if (myItem.parent()):
            useContact  = 1
            myAddrID    = myItem.parent().tag
            myContactID = myItem.tag
        else:
            useContact = 0
            myAddrID = myItem.tag

        if (useContact == 1):
            quContacts = kumula.DB.doSelect("clients_contacts", ("title","name","forename"), "", {"id":myContactID})
            while quContacts.next():
                aContactTitle    = quContacts.value(0).toString()
                aContactName     = quContacts.value(1).toString()
                aContactForename = quContacts.value(2).toString()
        else:
                aContactTitle    = QString("")
                aContactName     = QString("")
                aContactForename = QString("")

        quAddresses = kumula.DB.doSelect("clients_addresses", ("title","addressee","adcomment","address","zip","city","district"), "", {"id":myAddrID})
        while quAddresses.next():
            myText = self.selectedTemplate().formatReceiver(
                        quAddresses.value(0).toString(),
                        quAddresses.value(1).toString(),
                        quAddresses.value(2).toString(),
                        quAddresses.value(3).toString(),
                        quAddresses.value(4).toString(),
                        quAddresses.value(5).toString(),
                        quAddresses.value(6).toString(),
                        useContact, aContactTitle, aContactName, aContactForename)
            self.edReceiver.setText(myText)


    def updateFormatButtons(self):
        self.actFormatBold.setOn(self.edBody.bold(), 1)
        self.actFormatItalic.setOn(self.edBody.italic(), 1)
        self.actFormatUnderlined.setOn(self.edBody.underline(), 1)



    def __tr(self,s,c = None):
        return qApp.translate("MainWindow",s,c)


    def handleTabChanged(self, widget=None):
        if (self.tabMain.currentPageIndex() == 2):
            self.edBody.setFocus()
            self.actEditUndo.setEnabled(1)
            self.actEditRedo.setEnabled(1)
            self.actEditCut.setEnabled(1)
            self.actEditCopy.setEnabled(1)
            self.actEditPaste.setEnabled(1)
            self.actEditDelete.setEnabled(1)
            self.actEditSelectAll.setEnabled(1)
            self.actFormatBold.setEnabled(1)
            self.actFormatItalic.setEnabled(1)
            self.actFormatUnderlined.setEnabled(1)
            self.actFormatTextColor.setEnabled(1)
        else:
            self.actEditUndo.setEnabled(0)
            self.actEditRedo.setEnabled(0)
            self.actEditCut.setEnabled(0)
            self.actEditCopy.setEnabled(0)
            self.actEditPaste.setEnabled(0)
            self.actEditDelete.setEnabled(0)
            self.actEditSelectAll.setEnabled(0)
            self.actFormatBold.setEnabled(0)
            self.actFormatItalic.setEnabled(0)
            self.actFormatUnderlined.setEnabled(0)
            self.actFormatTextColor.setEnabled(0)



    def handleFileNew(self):
        kumula.runKumulaApp(kumula.ProgramName, {"client":kumula.ClientID})


    def handleFileNewClient(self):
        kumula.runKumulaApp(kumula.ProgramName)


    def handlePdfShow(self):
        # Check the availability of the PDF library from www.ReportLab.com!!!
        try:
            from reportlab.pdfgen import canvas
        except:
            result = QMessageBox.critical(kumula.MainWin, "Library missing",
                "To generate a pdf file, you need to have the <b>ReportLab PDF Library</b> installed.",
                "&OK")
            return

        # Generate temporary file...
        pdfFile = tempfile.mktemp() + ".pdf"
        self.tempFiles = self.tempFiles + (pdfFile,)

        # ...and create a PDF!
        self.template = self.selectedTemplate()
        self.template.setReceiver(QStringList.split("\n", self.edReceiver.text(), 1))

        # THESE ARE JUST DEMO DATA UNTIL THE
        # USER INTERFACE ALLOWS INPUT!!! ---------------
        self.template.setInfo("subject",  "Demo letter")
        self.template.setInfo("yourdate", "2005-12-31")
        self.template.setInfo("yoursign", "ab/cd")
        self.template.setInfo("oursign",  "xyz")
        self.template.setInfo("ourdate",  "2006-01-01")
        # ----------------------------------------------

        self.template.buildLetter(pdfFile, self.edBody.text())
        kumula.runExternalApp(kumula.AppPDF, pdfFile)


# WHY DEFINE OWN HANDLE METHODS INSTEAD OF CONNECTING THE ACTION TO self.edBody DIRECTLY?
# => self.edBody is created in self.createUI(), but the actions are connected in self.createMenuBar() BEFORE!

    def handleEditUndo(self):
        self.edBody.undo()

    def handleEditRedo(self):
        self.edBody.redo()

    def handleEditCut(self):
        self.edBody.cut()

    def handleEditCopy(self):
        self.edBody.copy()

    def handleEditPaste(self):
        self.edBody.paste()

    def handleEditDelete(self):
        self.edBody.delChar()

    def handleEditSelectAll(self):
        self.edBody.selectAll()

    def handleFormatBold(self):
        self.edBody.setBold(self.actFormatBold.isOn())

    def handleFormatItalic(self):
        self.edBody.setItalic(self.actFormatItalic.isOn())

    def handleFormatUnderlined(self):
        self.edBody.setUnderline(self.actFormatUnderlined.isOn())

    def handleFormatTextColor(self):
        pass

